//+------------------------------------------------------------------+
//|                        Smoothed Rsi Inverse Fisher Transform.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1  DeepSkyBlue
#property indicator_color2  PaleVioletRed
#property indicator_color3  PaleVioletRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_minimum -5
#property indicator_maximum 105
#property indicator_level1  12
#property indicator_level2  88
#property indicator_levelcolor DimGray

//
//
//
//
//
extern string TimeFrame       = "Current time frame";
extern int    RsiPeriod       = 4;
extern int    EmaPeriod       = 4;
extern int    RwmaLength      = 2;
extern int    Depth           = 10;
extern int    Price           = PRICE_CLOSE;
extern bool   Interpolate     = true;
extern bool   MultiColor      = true;

extern bool   alertsOn        = true;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

#define MAX_depth 50
double  inv[];
double  invUa[];
double  invUb[];
double  rwm[];
double  ema1[];
double  ema2[];
double  trend[];
double  pBuffer[][MAX_depth];




string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
      IndicatorBuffers(7);
      SetIndexBuffer(0,inv);
      SetIndexBuffer(1,invUa);
      SetIndexBuffer(2,invUb);
      SetIndexBuffer(3,rwm);
      SetIndexBuffer(4,ema1);
      SetIndexBuffer(5,ema2);
      SetIndexBuffer(6,trend);
      
      //
      //
      //
      //
      //
      
      RwmaLength = MathMax(RwmaLength,2);
      Depth      = MathMax(MathMin(Depth,MAX_depth),2);
      
      indicatorFileName   = WindowExpertName();
      returnBars          = TimeFrame=="returnBars";          if (returnBars)          return(0);
      calculateValue      = TimeFrame=="calculateValue";      if (calculateValue)      return(0);
         
         
         
      //
      //
      //
      //
      //
   timeFrame = stringToTimeFrame(TimeFrame);            
   IndicatorShortName(timeFrameToString(timeFrame)+ " Rwma Smoothed Rsi Inverse Fisher Transform ("+RsiPeriod+","+EmaPeriod+","+RwmaLength+","+Depth+")");
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,l,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=Bars-counted_bars;
           if (returnBars) { inv[0] = limit+1; return(0); }
           if (ArrayRange(pBuffer,0) != Bars) ArrayResize(pBuffer,Bars);

   //
   //
   //
   //
   //
   if (calculateValue || timeFrame == Period())
   {
   
   if (MultiColor && !calculateValue && trend[limit]==-1) CleanPoint(limit,invUa,invUb);
   
   
   
   double weight = (RwmaLength+1.0)*RwmaLength*0.5;
      for(i=limit, r=Bars-limit-1; i>=0; i--,r++)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      
         //
         //
         //
         //
         //

         double coeff  = 5;
         double sum    = 0;
         double sumc   = 0;
         for (int k=0; k<Depth; k++)
         {
            pBuffer[r][k] = price;
            if (r>RwmaLength)
            {
               for (l=0, price=0; l<RwmaLength; l++) price += (RwmaLength-l)*pBuffer[r-l][k];
                                                     price /= weight;
            }
            sum  += coeff*price;
            sumc += coeff;
            coeff = MathMax(coeff-1,1);
         }
         rwm[i] = sum/sumc;
      }
   
      //
      //
      //
      //
      //
   
      double alpha = 2.0 / (1+EmaPeriod);
      for(i=limit; i>=0; i--)
      {
         double rsi       = 0.1 * (iRSIOnArray(rwm,0,RsiPeriod,i)-50.0);
                ema1[i]   = ema1[i+1]+alpha*(rsi    -ema1[i+1]);
                ema2[i]   = ema2[i+1]+alpha*(ema1[i]-ema2[i+1]);
         double zlema     = 2.0*ema1[i]-ema2[i];
                inv[i]    = 50.0*(1+(MathExp(2*zlema)-1)/(MathExp(2*zlema)+1));
                invUa[i]  = EMPTY_VALUE;
                invUb[i]  = EMPTY_VALUE;
                trend[i]  = trend[i+1];
 
               if (inv[i] > inv[i+1]) trend[i]= 1;
               if (inv[i] < inv[i+1]) trend[i]=-1;
               if (MultiColor && !calculateValue && trend[i]==-1) PlotPoint(i,invUa,invUb,inv);
      }      

   //
   //
   //
   //
   //
   manageAlerts();      
   return(0);
}

   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (MultiColor && trend[limit]==-1) CleanPoint(limit,invUa,invUb);
   for (i=limit;i>=0; i--)
   {
   int y    = iBarShift(NULL,timeFrame,Time[i]);
   inv[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,EmaPeriod,RwmaLength,Depth,Price,0,y);
   invUa[i] = EMPTY_VALUE;
   invUb[i] = EMPTY_VALUE;
   trend[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RsiPeriod,EmaPeriod,RwmaLength,Depth,Price,6,y);
   
   
   if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         

         datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int j = 1; j < n; j++)
         {
  	      inv[i+j] = j*factor*inv[i+n] + (1.0-j*factor)*inv[i];
  	      
  	      }  	            
     }
     if (MultiColor) for (i=limit;i>=0;i--) if (trend[i]==-1) PlotPoint(i,invUa,invUb,inv);
     
      manageAlerts(); 
    
   return(0);
}

//+------------------------------------------------------------------+


void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Rwma Rsi Inv Fisher changed direction to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Rwma Rsi Inv Fisher "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}
  

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}